package com.ningdatech.pmapi.projectdeclared.controller;

import com.ningdatech.basic.model.PageVo;
import com.ningdatech.pmapi.projectdeclared.entity.dto.DeclaredProjectDto;
import com.ningdatech.pmapi.projectdeclared.entity.dto.DeclaredProjectListParamDto;
import com.ningdatech.pmapi.projectdeclared.entity.dto.ProjectDraftSaveDto;
import com.ningdatech.pmapi.projectdeclared.entity.vo.ProjectDeclaredDetailVO;
import com.ningdatech.pmapi.projectdeclared.entity.vo.ProjectDeclaredListItemVO;
import com.ningdatech.pmapi.projectdeclared.entity.vo.ProjectDraftVo;
import com.ningdatech.pmapi.projectdeclared.manage.DeclaredProjectManage;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 * @Classname DeclaredProjectController
 * @Description 申报项目
 * @Date 2023/1/31 11:29
 * @Author PoffyZhang
 */
@Slf4j
@Validated
@RestController
@RequestMapping("/api/v1/declared")
@Api(value = "DeclaredProject", tags = "申报管理-申报项目")
@RequiredArgsConstructor
public class DeclaredProjectController {
    private final DeclaredProjectManage declaredProjectManage;

    @ApiOperation(value = "申报项目已申报列表", notes = "申报项目已申报列表")
    @GetMapping("/list")
    public PageVo<ProjectDeclaredListItemVO> list(@Validated @ModelAttribute DeclaredProjectListParamDto params) {;
        return declaredProjectManage.page(params);
    }

    @ApiOperation(value = "申报项目已申报项目", notes = "申报项目已申报项目")
    @GetMapping("/{id}")
    public ProjectDeclaredDetailVO detail(@PathVariable Long id) {;
        return declaredProjectManage.detail(id);
    }

    @ApiOperation(value = "申报项目草稿箱列表", notes = "申报项目草稿箱列表")
    @GetMapping("/draft")
    public PageVo<ProjectDraftVo> draft(@Validated @ModelAttribute DeclaredProjectListParamDto params) {
        return declaredProjectManage.pageDraft(params);
    }

    @ApiOperation(value = "申报项目草稿箱详情", notes = "申报项目草稿箱详情")
    @GetMapping("/draft/{id}")
    public ProjectDraftVo draft(@PathVariable Long id) {
        return declaredProjectManage.draftDatail(id);
    }

    @ApiOperation(value = "申报项目保存至草稿箱", notes = "申报项目保存至草稿箱")
    @PostMapping("/save-to-draft")
    public Object saveToDraft(@Validated @RequestBody ProjectDraftSaveDto dto) {
        return declaredProjectManage.saveToDraft(dto);
    }

    @ApiOperation(value = "申报项目", notes = "申报项目")
    @PostMapping("/start")
    public String startTheProcess(@Validated @RequestBody DeclaredProjectDto dto) {
        String instanceId = declaredProjectManage.startTheProcess(dto);
        return "启动流程实例 " + instanceId + " 成功";
    }
}
